#include "CrashLogger.h"

/////////////////////// WINDOWS ///////////////////////
#ifdef Q_OS_WIN

int sehFilter(_EXCEPTION_POINTERS *ex)
{
    std::fstream file;
    QDateTime timestamp = QDateTime::currentDateTimeUtc();

    QString fileName =
        "crash_log_" + timestamp.toString("dd.MM.yyyy_hh.mm.ss") + ".txt";

    file.open(fileName.toStdString(), std::fstream::out);
    writeException(file, ex);
    file.close();

    return EXCEPTION_EXECUTE_HANDLER;
}

void writeException(std::fstream &file, _EXCEPTION_POINTERS *ex)
{
    std::string date =
        QDateTime::currentDateTimeUtc().toString(Qt::ISODate).toStdString();
    file << "~~ Exception " << ex->ExceptionRecord->ExceptionAddress << " at "
         << date << " ~~\n";
    writeStack(file, ex);
}

void writeStack(std::fstream &file, _EXCEPTION_POINTERS *ex)
{
    constexpr int MaxNameLen = 256;

    BOOL retrieved;
    HANDLE process;
    HANDLE thread;
    HMODULE hModule;

    STACKFRAME64 stack;
    ULONG frame;
    DWORD64 displacement;

    DWORD disp;
    IMAGEHLP_LINE64 *line;

    char buffer[sizeof(SYMBOL_INFO) + MAX_SYM_NAME * sizeof(TCHAR)];
    char module[MaxNameLen];
    auto pSymbol = (PSYMBOL_INFO)buffer;

    CONTEXT ctxCopy;
    memcpy(&ctxCopy, ex->ContextRecord, sizeof(CONTEXT));
    memset(&stack, 0, sizeof(STACKFRAME64));

    process = GetCurrentProcess();
    thread  = GetCurrentThread();

#if !defined(_M_AMD64)
    stack.AddrPC.Offset    = (*ex->ContextRecord).Eip;
    stack.AddrPC.Mode      = AddrModeFlat;
    stack.AddrStack.Offset = (*ex->ContextRecord).Esp;
    stack.AddrStack.Mode   = AddrModeFlat;
    stack.AddrFrame.Offset = (*ex->ContextRecord).Ebp;
    stack.AddrFrame.Mode   = AddrModeFlat;
#endif

    // Initialize symbol handler for current process
    SymInitialize(process, NULL, true);

    for (frame = 0;; frame++)
    {
        retrieved = StackWalk64(
#if defined(_M_AMD64)
            IMAGE_FILE_MACHINE_AMD64,
#else
            IMAGE_FILE_MACHINE_I386,
#endif
            process,
            thread,
            &stack,
            &ctxCopy,
            NULL,
            SymFunctionTableAccess64,
            SymGetModuleBase64,
            NULL);
        if (!retrieved)
            break;

        pSymbol->SizeOfStruct = sizeof(SYMBOL_INFO);
        pSymbol->MaxNameLen   = MaxNameLen;
        SymFromAddr(process,
                    (ULONG64)stack.AddrPC.Offset,
                    &displacement,
                    pSymbol);

        line               = (IMAGEHLP_LINE64 *)malloc(sizeof(IMAGEHLP_LINE64));
        line->SizeOfStruct = sizeof(IMAGEHLP_LINE64);

        // Try to get line
        if (SymGetLineFromAddr64(process, stack.AddrPC.Offset, &disp, line))
        {
            file << "\tat " << pSymbol->Name << " in " << line->FileName
                 << ": line: " << line->LineNumber
                 << " address: " << pSymbol->Address << "\n";
        }
        else
        {
            // Failed to get line
            file << "\tat" << pSymbol->Name << ", address: " << pSymbol->Address
                 << "\n";
            hModule = NULL;
            lstrcpyA(module, "");
            GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS |
                                  GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT,
                              (LPCTSTR)(stack.AddrPC.Offset), &hModule);

            // At least print module name
            if (hModule != NULL)
                GetModuleFileNameA(hModule, module, MaxNameLen);

            file << "in " << module << "\n";
        }

        free(line);
        line = NULL;
    }
}
/////////////////////// LINUX & MACOS ///////////////////////
#elif Q_OS_LINUX || Q_OS_MAC

/*
The code should work(untested) until the todo comment.

void exceptionHandler(int sig, struct sigcontext ctx)
{
    QDateTime timestamp = QDateTime::currentDateTimeUtc();
    QString fileName    = QString("crash_log_") +
                       timestamp.toString("dd.MM.yyyy_hh.mm.ss") + ".txt";

    auto isoDate = timestamp.toString(Qt::DateFormat::ISODate);
    auto sigStr  = std::to_string(sig);
    int fd       = open(fileName.toStdString().c_str(), O_WRONLY | O_CREAT);

    // Generate file with current timestamp
    std::fstream file;
    void *trace[50];
    char **messages = (char **)NULL;
    int i, traceSize = 0;

    write(fd, "~~ Exception at ", 16);
    write(fd, isoDate.toStdString().c_str(), isoDate.length());
    write(fd, " , signal: ", 11);
    write(fd, sigStr.c_str(), sigStr.length());
    write(fd, " ~~", 3);

    // get void*'s for all entries on the stack
    traceSize = backtrace(trace, 50);

    // overwrite sigaction with caller's address
    trace[1] = (void *)ctx.eip;
    messages = backtrace_symbols(trace, trace_size);

    for (i = 1; i < traceSize; i++)
    {
        auto iStr       = std::to_string(i);
        auto messageStr = messages[i];
        auto messageLen = strlen(messageStr);

        write(fd, iStr.c_str(), iStr.length());
        write(fd, ": ", 2);
        write(fd, messageStr, messageLen);

        /* find first occurrence of '(' or ' ' in message[i] and assume
         * everything before that is the file name. (Don't go beyond 0 though
         * (string terminator)*/
        size_t p = 0;
        while (messages[i][p] != '(' && messages[i][p] != ' ' &&
               messages[i][p] != 0)
            ++p;

        // TODO: change the following code, as we cannot retrieve
        // the line by running addr2line in the terminal.
        // See: https://stackoverflow.com/questions/11556321/how-do-i-access-the-addr2line-functionality-within-my-c-program

        char syscom[256];
        sprintf(syscom, "addr2line %p -e %.*s", trace[i], p, messages[i]);
        // last parameter is the file name of the symbol
        system(syscom);
    }

    close(fd);
}

*/

#endif