#pragma once

#include <fstream>
#include <sstream>

#include <QDateTime>

/////////////////////// WINDOWS ///////////////////////
#ifdef Q_OS_WIN

#include <QDebug>
#include <Windows.h>
#include "winnt.h"
#include <process.h>
#include <DbgHelp.h>
#include <format>

/**
 * Structured Exception Handling (SEH) filter. Used together with __try
 * and __catch. See: https://learn.microsoft.com/en-us/cpp/cpp/try-except-statement?view=msvc-170
 */
int sehFilter(_EXCEPTION_POINTERS *ex);
/**
 * Write an exception information and a stacktrace to a file.
 * @param file The file to write to.
 * @param ex Exception info pointer (winnt.h).
 */
void writeException(std::fstream  &file, _EXCEPTION_POINTERS *ex);
/**
 * Write an exception stacktrace to the file. Used by writeException.
 * @param file The file to write to.
 * @param ex Exception info pointer (winnt.h).
 */
void writeStack(std::fstream &file, _EXCEPTION_POINTERS *ex);

/////////////////////// LINUX & MACOS ///////////////////////
#elif Q_OS_LINUX || Q_OS_MAC

/**
 * TODO: add linux support
 *
 * #include <io.h>
 * #include <stdio.h>
 * #include <execinfo.h>
 * #include <signal.h>
 * #include <stdlib.h>
 * #include <unistd.h>
 * #include <fcntl.h>
 *
 * void exceptionHandler(int sig);
 */

#endif