/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "MessageBroker.h"

#include <Core/Module/Module.h>

void MessageBroker::subscribe(Filter filter, Module* observer,
                              Callback callback)
{
    observers.insert(filter, {observer, new Callback(std::move(callback))});

    // Register the beforeDelete slot of the module to automatically remove the
    // subscriber
    connect(observer->getEventHandler(), &EventHandler::beforeDelete, this,
            &MessageBroker::onModuleDeleted);
}

void MessageBroker::unsubscribe(Filter filter, Module* module)
{
    auto callbackList = observers.values(filter);

    for (auto& elem : callbackList)
    {
        if (elem.first == module)
            observers.remove(filter, elem);
    }
}

void MessageBroker::publish(const Message& message)
{
    for (auto filter : observers.keys())
    {
        // If the message matches the filed, then notify all its subscribers
        if (filter.match(message))
        {
            for (auto subscriber : observers.values(filter))
            {
                auto copy = message;
                filter.filterMessage(copy);
                (*subscriber.second)(copy, filter);
            }
        }
    }
}

void MessageBroker::onModuleDeleted(Module* module)
{
    for (auto filter : observers.keys())
    {
        for (auto subscriber : observers.values(filter))
        {
            if (subscriber.first == module)
            {
                observers.remove(filter, subscriber);
                delete subscriber.second;
            }
        }
    }
}
