/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>
#include <Core/ModulesManager/ModulesManager.h>

#include <QMap>
#include <QObject>
#include <QSerialPort>

#include "MavlinkVersionHeader.h"
#include "MavlinkWriter.h"

class MavlinkModule;

/*
 * This class translate commands received from the UI in mavlink messages that
 * can be sent.
 */
class MavlinkCommandAdapter : public QObject
{
    Q_OBJECT

public:
    MavlinkCommandAdapter();

    void send(mavlink_message_t msg);
    void setSerialPort(QSerialPort* value);
    bool encodeCommand(const Message& msg, mavlink_message_t& output);

    bool produceMsgFromXml(const XmlObject& xml, mavlink_message_t* msg);
    static const int MAV_CMP = 96;
    static const int MAV_SYS = 171;

private:
    QSerialPort* serial = nullptr;

    MavlinkWriter mavlinkWriter;
    QMap<QString, int> msgNameIdMap;
};
