/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QList>
#include <QProcess>
#include <QString>

#include "Core/XmlObject.h"
#include "DeployerPathPicker.h"

class Deployer
{
public:
    Deployer();

    bool deploy(const QList<QString> &activeModules);
    QString getDeployReleaseFolder() const;

protected:
    QString writeBatFile() const;
    void copyPath(const QString &src, const QString &dst);

    QList<QString> loadFile(const QString &path);
    QString getProFileName() const;
    QString getSourceDir() const;
    QString getNewSourceDir() const;
    QString getNewProFilePath() const;
    QString getModuleListFilePath() const;

    void makeDeployDir() const;

    bool writeFile(const QList<QString> &text, const QString &path);
    void setupModulesFiles(const QList<QString> &activeModules);
    bool checkIfFilesIsActive(const QString &line) const;

    void produceNewProFile();
    QList<QString> makeNewProFile(const QList<QString> &oldProFile) const;

    void produceNewModuleListFile();
    QList<QString> makeNewModulesListFile(
        const QList<QString> &oldModulesListFile) const;

    bool isDeployCompletedSuccessfully();
    bool showResultMessage();

    bool isDeployConfirmed();

private:
    QList<QString> modulesFiles;
    QString mingw81_64_bin          = "C:/Qt/5.15.1/mingw81_64/bin";
    QString mingw810_64_bin         = "C:/Qt/Tools/mingw810_64/bin";
    QString relativeModuleFilesPath = "/Modules/moduleslist.cpp";

    QString proFilePath              = "";
    QString deployDir                = "";
    QString newCopySourceFolderName  = "source";
    QString newCopyReleaseFolderName = "release";

    QString proFileRestartCopyTag = "qnx:";
    QString proFileEndCopyTag     = "SOURCES";

    DeployerPathPicker deployerPicker;
};
