/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "PrefabDialog.h"

#include "ui_PrefabDialog.h"

PrefabDialog::PrefabDialog(QWidget *parent)
    : QDialog(parent), ui(new Ui::PrefabDialog)
{
    ui->setupUi(this);
    this->setFixedHeight(800);
    this->setMinimumWidth(800);
    connect(ui->create_pushButton, &QPushButton::clicked, this,
            &PrefabDialog::onCreate);
    this->setStyleSheet("background-color: #212121 ;");
    QString groupboxStyle =
        "QGroupBox {"
        "   border:1px solid #00bfff;"
        "  border-radius: 7px;"
        " margin-top: 2ex;"
        "}"

        "QGroupBox::title {"
        "   subcontrol-origin: margin;"
        "  subcontrol-position: top center;"
        " padding-left: 10px;"
        "padding-right: 10px;"
        "color: #00bfff;"
        "}";
    ui->title_groupBox->setStyleSheet(groupboxStyle);
    ui->caption_groupBox->setStyleSheet(groupboxStyle);
    ui->description_groupBox->setStyleSheet(groupboxStyle);

    QString lineeditStyle =
        "QLineEdit"
        "{"
        "   background-color: #2d2d2d;"
        "  padding: 2px;"
        " border-style: solid;"
        "border: 1px solid #00bfff;"
        "border-radius: 3px;"
        "color: #00bfff;"
        "}";

    ui->title_lineEdit->setStyleSheet(lineeditStyle);
    ui->caption_lineEdit->setStyleSheet(lineeditStyle);

    QString palaintextStyle =
        "QPlainTextEdit"
        "{"
        /*background-color: #201F1F;;
        color: silver;
        border-radius: 3px;
        border: 1px solid #3A3939;*/

        "   background-color: #2d2d2d;"
        "  padding: 2px;"
        " border-style: solid;"
        "border: 1px solid #00bfff;"
        "border-radius: 3px;"
        "color: #00bfff;"
        "}";
    ui->Desc_plainTextEdit->setStyleSheet(palaintextStyle);

    ui->create_pushButton->setStyleSheet(
        ""
        "QPushButton"
        "{"
        "background-color: none;"
        "border: 2px solid #00bfff;"
        "color: #00bfff;"
        "border-radius: 3px;"
        "}"
        "QPushButton:hover"
        "{"
        "background-color: #474747;"
        "border: 2px solid #00bfff;"
        "color: #00bfff;"
        "}"
        "QPushButton:pressed {  background-color: QLinearGradient( x1: 0, y1: "
        "0, x2: 0, y2: 1, stop: 0 #506987, stop: 1 #00739a);}");
}

PrefabDialog::~PrefabDialog() { delete ui; }

void PrefabDialog::onCreate()
{
    QString title       = ui->title_lineEdit->text();
    QString caption     = ui->caption_lineEdit->text();
    QString description = ui->Desc_plainTextEdit->toPlainText();
    emit newPrefabCreated(title, caption, description);
    this->close();
}
