/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QWidget>

#include "Core/XmlObject.h"

namespace Ui
{
class PrefabViewElement;
}

class PrefabViewElement : public QWidget
{
    Q_OBJECT

public:
    explicit PrefabViewElement(const QString &name, QWidget *parent = nullptr);
    ~PrefabViewElement();

    void setName(const QString &value);
    QString getName() const;

    QString getDesctiption() const;
    void setDesctiption(const QString &value);

    QString getPathToImg() const;
    void setPathToImg(const QString &value);

    XmlObject getXml() const;
    void setXml(XmlObject &xml);

    void addActiveModuleName(const QString &moduleName);
    void removeActiveModuleName(const QString &moduleName);
    bool containsModule(const QString &moduleName);

    void setDeleteButtonVisibility(bool visibility);
    void setLaunchButtonText(const QString &txt);
    void setIconButtonVisibility(bool visibility);

    QString getFilePath() const;
    void setFilePath(const QString &value);

public slots:
    void onIconClicked();
    void onLaunchClicked();
    void onDeleteClicked();

signals:
    void selected(PrefabViewElement *prefabVElement);
    void launchRequested(PrefabViewElement *prefabVElement);
    void deleteRequested(PrefabViewElement *prefabVElement);

private:
    Ui::PrefabViewElement *ui;

    QString name;
    QString desctiption;
    QString pathToImg;
    XmlObject xml;
    QList<QString> activeModules;
    QString filePath = "";

    int height = 0;
    int width  = 0;
};
