/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QCheckBox>
#include <QVBoxLayout>
#include <QWidget>

#include "Modules/DefaultModule/DefaultModule.h"

class PrefabViewElement;

namespace Ui
{
class SkywardHubModule;
}

class SkywardHubModule : public DefaultModule
{
    Q_OBJECT

public:
    explicit SkywardHubModule(QWidget *parent = nullptr);
    ~SkywardHubModule();

    QWidget *toWidget() override;

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &xmlObject) override;

    QCheckBox *createPanelsCheckbox(const QString &txt);
    void insertCheckBox(QCheckBox *checkBox);

public slots:
    PrefabViewElement *createNewPrefab(XmlObject &xmlPrefab);
    void dialogRequested();

    void onPrefabSelected(PrefabViewElement *element);
    void onPrefabLaunchRequested(PrefabViewElement *element);
    void onPrefabDeployRequested(PrefabViewElement *element);
    void onPrefabDeleteRequested(PrefabViewElement *element);
    void onCheckBoxClicked(QCheckBox *cBox);

protected:
    void fillCompleteModulesList();
    void loadPrefabs();
    void updateActiveCheckboxList(PrefabViewElement *element);
    void initUsedModules(PrefabViewElement *element,
                         const XmlObject &root) const;
    void addEmptyPrefab();
    void createAndInsertPrefabFromXml(XmlObject &xml,
                                      const QString &filePath = "");
    void addPrefab(PrefabViewElement *element);
    void insertPrefabInUi(PrefabViewElement *element);
    void clearPrefab();

private:
    Ui::SkywardHubModule *ui;
    QList<QString> panelsList;
    QList<QCheckBox *> completeModule_checkboxList;

    QList<PrefabViewElement *> prefabs;
    PrefabViewElement *selectedElement = nullptr;
    PrefabViewElement *emptyPrefab     = nullptr;
    int nextColumn                     = 0;
    int nextRow                        = 0;
    int maxColum                       = 5;
};
