/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QList>
#include <QMap>
#include <QString>

namespace StatesList
{

// Rockets states
enum class State : int
{
    INVALID = 0,
    INIT,
    INIT_ERROR,
    SENSORS_CALIBRATION,
    ALGOS_CALIBRATION,
    DISARMED,
    TEST_MODE,
    ARMED,
    ASCENDING,
    DROGUE_DESCENT,
    TERMINAL_DESCENT,
    LANDED,
};

// Groundstation labels
static const QList<QString> statesLabels{
    "INIT",      "TEST\nMODE",      "SENSORS\nCALIBRATION", "DISARMED", "ARMED",
    "ASCENDING", "DROGUE\nDESCENT", "TERMINAL\nDESCENT",    "LANDED",
};

// Map from rocket states to groundstation labels
static const QMap<State, int> statesIndexes{
    {State::INIT, 0},
    {State::INIT_ERROR, 0},
    {State::TEST_MODE, 1},
    {State::SENSORS_CALIBRATION, 2},
    {State::ALGOS_CALIBRATION, 2},
    {State::DISARMED, 3},
    {State::ARMED, 4},
    {State::ASCENDING, 5},
    {State::DROGUE_DESCENT, 6},
    {State::TERMINAL_DESCENT, 7},
    {State::LANDED, 8},
};

}  // namespace StatesList